@@ -5,11 +5,12 @@ import numpy as np
 import pandas as pd
 from loguru import logger
 from .config import Style
-from .utils import PickleUtils, Rosetta, as_list
-import matplotlib as mpl
-import seaborn as sns
+from .utils import PickleUtils, Rosetta
+from .data_quality import basis_quality_checks, basis_quality_plots, fill_ratio
 from copy import deepcopy
 import statsmodels.api as sm
+from .plotting import visualize_x_y, visualize_1d_distribution, visualize_surface
+import pathlib
 
 
 class VectorMultiset(PickleUtils, Style, Rosetta):
@@ -31,7 +32,7 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         :param cumulative: whether to apply a cumulative sum (default = False)
         :return: Extracted data
         """
-        values: pd.Series = self.data[feature]
+        values: pd.Series = self.data.loc[:, feature]
         if args:
             for unpack_next in args:
                 values: List[Any] = [x.__getattribute__(unpack_next) for x in values]
@@ -50,7 +51,7 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         """
         return self.data
 
-    def from_dataframe(self, data_frame: pd.DataFrame, inplace: bool = True, **kwargs):
+    def from_dataframe(self, data_frame: pd.DataFrame, inplace: bool = True, **kwargs) -> Union[Any, None]:
         """ Makes an VectorMultiset from a pandas data frame
 
         :param data_frame: data frame to import
@@ -64,21 +65,20 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         else:
             return self.__class__(data=data_frame, **kwargs)
 
-    def to_csv(self, file_path: str = None) -> None:
+    def to_csv(self, file_path: Union[str, pathlib.Path] = None) -> None:
         """ Saves the data as a CSV file (note: this drops the name_root)
 
         :param file_path: path to write the file
         """
         if not file_path:
             file_path: str = f'default_filename_{time.time():.0f}.csv'
-        self.data.to_csv(file_path=file_path)
+        self.data.to_csv(file_path)
 
-    def read_csv(self, file_path: str, inplace: bool = True, **kwargs):
+    def read_csv(self, file_path: Union[str, pathlib.Path], inplace: bool = True, **kwargs) -> Union[Any, None]:
         """Reads data from a CSV file
 
         :param file_path: file to read
         :param inplace: import inplace (default) or return the result
-        :return:
         """
         data: pd.DataFrame = pd.read_csv(file_path)
         if inplace:
@@ -88,64 +88,46 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         else:
             return self.__class__(data=data, **kwargs)
 
+    def to_feather(self, file_path: Union[str, pathlib.Path], **kwargs) -> None:
+        """ Saves the data as a feather file (note: this drops the name_root). keyword
+
+        :param file_path: path to write the file
+        """
+        self.data.to_feather(file_path, **kwargs)
+
+    def read_feather(self, file_path: Union[str, pathlib.Path], inplace: bool = True, **kwargs) -> Union[Any, None]:
+        """Reads data from a CSV file
+
+        :param file_path: file to read
+        :param inplace: import inplace (default) or return the result
+        """
+        data: pd.DataFrame = pd.read_feather(file_path)
+        if inplace:
+            self.data = data
+            for key, value in kwargs.items():
+                self.__setattr__(key, value)
+        else:
+            return self.__class__(data=data, **kwargs)
+
     ################
     # Visualizations
     ################
 
-    def visualize_x_y(self, x_name: str, y_name: str, cumulative: str = '', figsize: Any = None, title: str = None,
-                      log_axes: str = '', types: Union[str, List[str]] = 'line', **kwargs) -> plt.Figure:
-        """ Creates a 2D (scatter and/or line) plot of x and y data
+    def visualize_x_y(self, x_name: str, y_name: str, cumulative: Union[str, List[str]] = '', **kwargs) -> plt.Figure:
+        """ Visualize in two dimensions
 
-        :param y_name: field or subfield to plot on x-axis
-        :param x_name: field or subfield to plot on y-axis
-        :param cumulative: 'x' or 'y' or 'xy' to plot cumulative data on that axis
-        :param figsize: figure size
-        :param title: figure title (raw strings are translated)
-        :param log_axes: 'x' or 'y' or 'xy' to plot that axis/axes on a log scale
-        :param types: any subset of ['scatter, 'plot']
-        :return: figure handle
+        :param x_name: name of the x-axis data feature
+        :param y_name: name of the y-axis data feature
+        :param cumulative: Which (if any) dimensions should be cumulative, e.g. 'x' or ['x','y'] or 'xy'
+        :param kwargs: additional kwargs for isthmuslib.visualize_x_y, passed through to matplotlib.pyplot.scatter()
+        :return: figure handle for the plot
         """
-
-        if not figsize:
-            figsize: Tuple[float, float] = self.figsize
-
-        # Data
         x_data: List[Any] = self.values(x_name, cumulative='x' in cumulative)
         y_data: List[Any] = self.values(y_name, cumulative='y' in cumulative)
-        x_label, y_label = (self.translate(item) for item in [x_name, y_name])
-        if 'x' in cumulative:
-            x_data: List[Any] = np.cumsum(x_data)
-            x_label += ' (cumulative)'
-        if 'y' in cumulative:
-            y_data: List[Any] = np.cumsum(y_data)
-            y_label += ' (cumulative)'
-
-        # Plot
-        figure_handle: plt.Figure = plt.figure(facecolor=self.facecolor, figsize=figsize)
-        if 'line' in (styles_list := as_list(types)):
-            plt.scatter(x_data, y_data, self.markersize, self.color, **kwargs)
-        if 'plot' in styles_list:
-            plt.plot(x_data, y_data, color=self.color, linewidth=self.linewidth)
-
-        # Labels
-        plt.xlabel(x_label, fontsize=self.label_fontsize)
-        plt.ylabel(y_label, fontsize=self.label_fontsize)
-        if title:
-            plt.title(title, fontsize=self.title_fontsize)
-        elif self.name_root:
-            plt.title(self.translate(self.name_root), fontsize=self.title_fontsize)
-        plt.grid(self.grid)
-
-        # Axes
-        if self.tight_axes:
-            plt.xlim([np.nanmin(x_data), np.nanmax(x_data)])
-            plt.ylim([np.nanmin(y_data), np.nanmax(y_data)])
-        if 'x' in log_axes:
-            plt.xscale('log')
-        if 'y' in log_axes:
-            plt.yscale('log')
-
-        return figure_handle
+        kwargs.setdefault('xlabel', self.translate(x_name))
+        kwargs.setdefault('ylabel', self.translate(y_name))
+        kwargs.setdefault('title', self.translate(self.name_root))
+        return visualize_x_y(x_data, y_data, style=Style(**self.dict()), **kwargs)
 
     def scatter(self, *args, **kwargs) -> plt.Figure:
         """ Creates a 2D scatter plot of x and y data (wraps visualize_x_y) """
@@ -161,85 +143,41 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
             raise ValueError(f"Plot requires x & y. Hint: You might want SlidingWindowResults.plot_results()")
         return self.visualize_x_y(types='plot', *args, **kwargs)
 
-    def hist(self, col_name: str, log_axes: str = '', title: str = None, **kwargs) -> plt.Figure:
+    def hist(self, col_name: str, **kwargs) -> plt.Figure:
         """ Plot a histogram of data. Useful kwargs: [cumulative, density, bins]
 
         :param col_name: name of field to plot
-        :param log_axes: 'y' to plot log-scale y-axis
-        :param title: figure title (raw strings are translated)
         :return: figure handle
         """
+        data: List[Any] = self.values(col_name)
+        kwargs.setdefault('xlabel', self.translate(col_name))
+        kwargs.setdefault('title', self.translate(self.name_root))
+        return visualize_1d_distribution(data, style=Style(**self.dict()), **kwargs)
 
-        # Plot
-        figure_handle: plt.Figure = plt.figure(facecolor=self.facecolor, figsize=self.figsize)
-        plt.hist((data := self.values(col_name)), color=self.color, **kwargs)
+    def surface(self, x_name: str, y_name: str, z_name: str, **kwargs) -> plt.Figure:
+        """ Plot a surface from the data. Useful kwargs: [cumulative, density, bins]
 
-        # Labels
-        if kwargs.get('cumulative'):
-            y_label: str = 'cumulative counts'
-        else:
-            y_label: str = 'counts'
-        if kwargs.get('density'):
-            y_label += " (density)"
-        plt.ylabel(y_label, fontsize=self.label_fontsize)
-        plt.xlabel(self.translate(col_name), fontsize=self.label_fontsize)
-        if title:
-            plt.title(title, fontsize=self.title_fontsize)
-        elif self.name_root:
-            plt.title(self.translate(self.name_root), fontsize=self.title_fontsize)
-
-        # Axes
-        if self.tight_axes:
-            plt.xlim([np.nanmin(data), np.nanmax(data)])
-        if 'y' in log_axes:
-            plt.yscale('log')
-        if 'x' in log_axes:
-            raise NotImplementedError("log x histogram requires different bin handling")
-
-        return figure_handle
-
-    def heatmap(self, x_name: str, y_name: str, z_name: str, y_axis_ascending: bool = True,
-                figsize: Any = None, title: str = None, **kwargs) -> plt.Figure:
-        """ Creates a heatmap from any 3 features of the vector set.
-
-        :param x_name: name of column to use for x-axis values
-        :param y_name: name of column to use for y-axis values
-        :param z_name: name of column to use for z-axis (color) values
-        :param y_axis_ascending: flip the y-axis to read low to high ascending (overrides weird sns default)
-        :param figsize: figsize
-        :param title: custom title
-        :param kwargs: sns.heatmap() kwargs (hint: annot=True adds labels)
-        :return: figure_handle
-
-        to-do: aggregation_function: Callable[[Any], float] for handling multiple z values at one (x,y) coordinate
+        :param x_name: name of the x-axis data feature
+        :param y_name: name of the y-axis data feature
+        :param z_name: name of the z-axis data feature (color)
+        :param kwargs: additional keyword arguments for visualize_surface which passes to seaborn heatmap()
+        :return: figure handle for the plot
         """
-
         if isinstance(self, VectorSequence) and (not x_name):
             x_name: str = self.basis_col_name
-        if not kwargs.get(figsize):
-            figsize: Any = self.figsize
+        kwargs.setdefault('xlabel', self.translate(x_name, missing_response='return_input'))
+        kwargs.setdefault('ylabel', self.translate(y_name, missing_response='return_input'))
+        kwargs.setdefault('title', self.translate(z_name, missing_response='return_input'))
+        return visualize_surface(self.data.loc[:, x_name], self.data.loc[:, y_name], self.data.loc[:, z_name], **kwargs)
 
-        # Make plot
-        figure_handle: plt.Figure = plt.figure(figsize=figsize, facecolor=self.facecolor)
-        df: pd.DataFrame = self.data.loc[:, [x_name, y_name, z_name]].pivot(y_name, x_name, z_name)
-        ax: plt.Axes = sns.heatmap(df, **kwargs)
+    def correlation_matrix(self, **kwargs) -> pd.DataFrame:
+        """
+        Very thin wrapper around correlation matrix (using pandas corr() method)
 
-        # Labels
-        plt.xlabel(self.translate(x_name), size=self.label_fontsize)
-        plt.ylabel(self.translate(y_name), size=self.label_fontsize)
-        if not title:
-            title: str = self.translate(z_name)
-            if isinstance(self, InfoSurface):
-                title: str = title.replace('_', ' ')  # so that we get "singular value 1" instead of "singular_value_1"
-            if self.name_root:
-                title += f" ({self.translate(self.name_root)})"
-        plt.title(title, size=self.title_fontsize)
-
-        # Axes
-        if y_axis_ascending:
-            ax.invert_yaxis()
-
-        return figure_handle
+        :param kwargs: additional keyword arguments for correlation_matrix (fed through to df.style.background_gradient)
+        :return: styled pandas dataframe
+        """
+        return correlation_matrix(self.data, **kwargs)
 
 
 class SlidingWindowResults(VectorMultiset):
@@ -249,55 +187,36 @@ class SlidingWindowResults(VectorMultiset):
 
     def group_by_window_width(self) -> Dict[float, pd.DataFrame]:
         """ Helper function that returns individual dataframes for each window width """
-        return {x: self.data[self.data['window_width'] == x] for x in set(self.data.window_width)}
+        return {x: self.data.loc[self.data.loc[:, 'window_width'] == x, :] for x in set(self.data.window_width)}
 
-    def plot_results(self, col_name: str, cumulative: bool = False, figsize: Any = None, title: str = None,
-                     legend_override: List[str] = None, **kwargs) -> plt.Figure:
+    def plot_results(self, col_name: str, legend_override: List[str] = None, **kwargs) -> plt.Figure:
         """ Plot any sequence (based on the data frame column name)
 
         :param col_name: dataframe column to plot
-        :param cumulative: plot cumulative values (default: False)
-        :param figsize: specify a custom figure size if desired
-        :param title: specify a custom title if desired
         :param legend_override: specify custom legend keys if desired
         :return: figure handle
         """
-        if not figsize:
-            figsize = self.figsize
+        x_arrays: List[Any] = []
+        y_arrays: List[Any] = []
+        labels: List[Any] = []
 
-        traces: List[mpl.collections.PathCollection] = []
-        labels: List[str] = []
-        figure_handle = plt.figure(figsize=figsize, facecolor=self.facecolor)
-
-        # For each `window_width` add a trace to the plot
+        # For each `window_width` add we'll add a trace to the plot
         group_by_window_width: Dict[float, pd.DataFrame] = self.group_by_window_width()
         for key in sorted(group_by_window_width.keys()):  # looping over group_by_window_width.items() gives wrong order
             value: pd.DataFrame = deepcopy(group_by_window_width[key])
-            value.sort_values(by=self.window_start_col_name, ascending=True, ignore_index=True)
-            y_vec: List[float] = value.loc[:, col_name]
-            if cumulative:
-                y_vec: List[float] = np.cumsum(y_vec)
-            h: mpl.collections.PathCollection = plt.scatter((x_vec := value.loc[:, self.window_start_col_name]), y_vec,
-                                                            self.markersize, **kwargs)
-            traces.append(h)
-            labels.append(f"Window width: {key}")
-            plt.plot(x_vec, y_vec, linewidth=self.linewidth)
+            value.sort_values(by=self.window_start_col_name, ascending=True, ignore_index=True, inplace=True)
+            x_arrays.append(value.loc[:, self.window_start_col_name])
+            y_arrays.append(value.loc[:, col_name])
+            labels.append(str(key))  # noqa: key is a string
 
-        # Labels
-        plt.xlabel('Window start', fontsize=self.label_fontsize)
-        plt.ylabel(self.translate(col_name), fontsize=self.label_fontsize)
-        if not title:
-            if self.name_root:
-                title: str = f"{self.translate_time(self.name_root)}: {self.translate(col_name)}"
-            else:
-                title: str = self.translate(col_name)
-        plt.title(title, fontsize=self.title_fontsize)
+        # Final settings and overrides
         if legend_override:
-            plt.legend(handles=traces, labels=legend_override, fontsize=self.legend_fontsize)
-        else:
-            plt.legend(handles=traces, labels=labels, fontsize=self.legend_fontsize)
-
-        return figure_handle
+            labels: List[str] = legend_override
+        kwargs.setdefault('types', ['scatter', 'plot'])
+        kwargs.setdefault('xlabel', self.translate(self.translate('window_start')))
+        kwargs.setdefault('ylabel', self.translate(col_name))
+        kwargs.setdefault('title', self.translate(self.name_root))
+        return visualize_x_y(x_arrays, y_arrays, legend_strings=labels, style=Style(**self.dict()), **kwargs)
 
     def plot_pdfs(self, col_name: str, legend_override: List[str] = None, **kwargs) -> plt.Figure:
         """ Plot the probability density function(s) of the sliding window results. Useful kwargs: [cumulative, density]
@@ -305,41 +224,28 @@ class SlidingWindowResults(VectorMultiset):
         :param col_name: which field should be plotted
         :param legend_override: custom legend labels if desired (must have same length as number of unique widths)
         :param kwargs: plt.hist() keyword arguments.
-]        :return: figure handle
+        :return: figure handle
         """
-        if not kwargs.get('alpha'):
-            kwargs['alpha'] = 0.5  # Transparency is helpful for reading overlapping histograms
-
-        # Make figure
-        figure_handle: plt.Figure = plt.figure(figsize=self.figsize, facecolor=self.facecolor)
         grouped: Dict[float, pd.DataFrame] = self.group_by_window_width()
+        data_sets: List[Any] = []
+        labels: List[Any] = []
         for key, value in grouped.items():
-            plt.hist(value[col_name], **kwargs)
-
-        # Legends
-        plt.xlabel(self.translate(col_name), fontsize=self.label_fontsize)
-        if kwargs.get("cumulative"):
-            y_label: str = 'cumulative counts'
-        else:
-            y_label: str = 'counts'
-        if kwargs.get("density"):
-            y_label += " (density)"
-        plt.ylabel(y_label, fontsize=self.label_fontsize)
-        plt.legend(grouped.keys())
+            data_sets.append(value[col_name])
+            labels.append(str(key))
         if legend_override:
-            plt.legend(legend_override, fontsize=self.legend_fontsize)
-        else:
-            plt.legend([f"{x} weeks " for x in grouped], fontsize=self.legend_fontsize)
-        return figure_handle
+            labels: List[str] = legend_override
+        kwargs.setdefault('xlabel', self.translate(col_name))
+        kwargs.setdefault('title', self.translate(self.name_root))
+        return visualize_1d_distribution(data_sets, legend_strings=labels, **kwargs)
 
     def heatmap_feature(self, col_name: str, **kwargs) -> plt.Figure:
-        return self.heatmap(self.window_start_col_name, self.window_width_col_name, col_name, **kwargs)
+        return self.surface(self.window_start_col_name, self.window_width_col_name, col_name, **kwargs)
 
 
 class InfoSurface(SlidingWindowResults):
     """ Wrapper for SlidingWindowResults that knows how to plot the infosurface """
 
-    def plot_info_surface(self, singular_values: List[int] = None) -> List[plt.Figure]:
+    def plot_info_surface(self, singular_values: List[int] = None, **kwargs) -> List[plt.Figure]:
         """ Plot the info surface showing value of singular vectors as a function of window start and width
 
         :param singular_values: Which singular values to plot [1, 2, 3] by default
@@ -349,19 +255,57 @@ class InfoSurface(SlidingWindowResults):
         if not singular_values:
             singular_values: List[int] = [1, 2, 3]
         for s in singular_values:
-            figure_handles.append(self.heatmap_feature(f"singular_value_{s}"))
+            figure_handles.append(self.heatmap_feature(f"singular_value_{s}", title=f'Singular value # {s}', **kwargs))
         return figure_handles
 
 
 class VectorSequence(VectorMultiset):
     """ Set of vectors in a sequence ordered according to some basis (e.g. height, wavelength, time) """
 
-    # (Inherits dataframe and name string from VectorMultiset)
+    # (Inherits `data` dataframe and `name_root` string from VectorMultiset)
     basis_col_name: str = 'basis'
+    error_if_basis_quality_issues: bool = False
 
     def __init__(self, **data: Any):
         super().__init__(**data)
-        self.sort()
+        if self.data is not None:
+            self.sort(by=self.basis_col_name, inplace=True, reset_index=True)
+            if self.error_if_basis_quality_issues:
+                is_ok, explanation = self.passes_basis_quality_checks()
+                if not is_ok:
+                    raise ValueError(f"Issues with basis:\n{explanation}\nSee: `error_if_basis_quality_issues`")
+
+    def basis_quality_checks(self) -> Tuple[bool, str]:
+        """ Checks basis data quality and returns both a True/False flag and a string with an explanation
+
+        :return: True if OK, and a string with explanation either way
+        """
+        return basis_quality_checks(self.data.loc[:, self.basis_col_name])
+
+    def passes_basis_quality_checks(self) -> bool:
+        """ Super thin wrapper around basis_quality_checks that drops the explanation string
+
+        :return: True if OK
+        """
+        result, _ = self.basis_quality_checks()
+        return result
+
+    def basis_quality_plots(self) -> List[plt.Figure]:
+        """ Creates a series of plots showing the basis data quality (missing data, nans, timeseries gaps, etc)
+
+        :return: figure handles for several plots
+        """
+        return basis_quality_plots(self.data.loc[:, self.basis_col_name])
+
+    def fill_ratio(self) -> float:
+        """ Checks for how many data points are observed for the expected number of data points in a uniform basis.
+            Attempts to infer spacing from median of diffs. If used on non-uniform data the result is not meaningful.
+
+        Use example: [10, 20, 40, 50] has 0.8 fill ratio because it is missing the 30 to complete the set
+
+        :return: fractional amount of the array
+        """
+        return fill_ratio(self.data.loc[:, self.basis_col_name])
 
     def slice(self, start_at: Any = None, stop_at: Any = None, inplace: bool = False, reset_index: bool = True):
         """ Slices the VectorSequence according to the basis
@@ -375,10 +319,10 @@ class VectorSequence(VectorMultiset):
         df: pd.DataFrame = deepcopy(self.data)
         df.sort_values(by=self.basis_col_name, ascending=True, inplace=True, ignore_index=True)
         if not start_at:
-            start_at: Any = min(df[self.basis_col_name])
+            start_at: Any = min(df.loc[:, self.basis_col_name])
         if not stop_at:
-            stop_at: Any = max(df[self.basis_col_name])
-        in_range: pd.DataFrame = df.loc[df[self.basis_col_name].between(start_at, stop_at), :]
+            stop_at: Any = max(df.loc[:, self.basis_col_name])
+        in_range: pd.DataFrame = df.loc[df.loc[:, self.basis_col_name].between(start_at, stop_at), :]
         if reset_index:
             in_range.reset_index(drop=True, inplace=True)
         if inplace:
@@ -388,7 +332,7 @@ class VectorSequence(VectorMultiset):
 
     def index_to_basis(self) -> None:
         """  Adds (or overwrites) 'basis' column with a default index [0, 1, 2, ...] """
-        self.data[self.basis_col_name] = list(range(len(self.data)))
+        self.data.loc[:, self.basis_col_name] = list(range(len(self.data)))
 
     def sort(self, by: str = None, inplace: bool = True, reset_index: bool = True):
         """ Sorts the data points by the basis
@@ -433,11 +377,11 @@ class VectorSequence(VectorMultiset):
             raise AttributeError(f'basis_col_name {self.basis_col_name} not a column in data frame:\n{self.data}')
 
         df: pd.DataFrame = pd.DataFrame()
-        basis: Tuple[float] = (self.data[self.basis_col_name].tolist())
+        basis: Tuple[float] = (self.data.loc[:, self.basis_col_name].tolist())
 
         if not window_widths:
-            data_duration: float = max(self.data[self.basis_col_name]) - min(self.data[self.basis_col_name])
-            window_widths: List[float] = [data_duration / x for x in range(20, 401, 20)]
+            duration: float = max(self.data.loc[:, self.basis_col_name]) - min(self.data.loc[:, self.basis_col_name])
+            window_widths: List[float] = [duration / x for x in range(20, 401, 20)]
 
         # Loop over window widths
         for i, window_width in enumerate(window_widths):
@@ -465,7 +409,7 @@ class VectorSequence(VectorMultiset):
                                 f"Starting at: {window_start} ({j + 1} of {len(window_starts)})")
 
         return SlidingWindowResults(window_width_col_name='window_width', window_start_col_name='window_start',
-                                    data=df)
+                                    data=df, name_root=self.name_root)
 
     def repackage(self, instance: Any, sequence_attribute: str = 'series', basis_name: str = 'basis') -> Any:
         cache: Dict[str, Any] = instance.dict()
@@ -487,9 +431,9 @@ class VectorSequence(VectorMultiset):
         except ValueError as e:
             raise ValueError(f"Error - did you include a non-numeric column, or not specify 'cols'? See: {e}")
 
-    def plot_decomposition(self, col: str, period: float, figsize: List[int] = None, which_plots: List[str] = None,
-                           xlabel: str = 'basis', ylabel: str = '[units]', title: str = None, xlim: List[float] = None,
-                           **kwargs) -> List[plt.Figure]:
+    def plot_decomposition(self, col: str, period: float, figsize: Tuple[float, float] = None,
+                           which_plots: List[str] = None, xlabel: str = 'basis', ylabel: str = '[units]',
+                           title: str = None, xlim: List[float] = None, **kwargs) -> List[plt.Figure]:
 
         """ Plot the seasonal (weekly, monthly, etc) decomposition. Must specify period
 
@@ -511,10 +455,10 @@ class VectorSequence(VectorMultiset):
         if not title:
             title: str = self.translate(self.name_root)
 
-        def add_labels(xlabel: str, ylabel: str) -> None:
-            plt.xlabel(xlabel, size=self.label_fontsize)
+        def add_labels(x_label: str, y_label: str) -> None:
+            plt.xlabel(x_label, size=self.label_fontsize)
+            plt.ylabel(y_label, size=self.label_fontsize)
             plt.legend(fontsize=self.legend_fontsize)
-            plt.ylabel(ylabel, size=self.label_fontsize)
 
         decomposition = self.seasonal_decompose(col, period, **kwargs)
         figure_handles: List[plt.Figure] = []
@@ -524,7 +468,7 @@ class VectorSequence(VectorMultiset):
             plt.plot(self.data.loc[:, self.basis_col_name], decomposition.observed, '-', label='Sequence',
                      color='black')
             add_labels(xlabel, ylabel)
-            plt.title(f'{title}  // observed', size=self.title_fontsize)
+            plt.title(f'{title}observed', size=self.title_fontsize)
             if xlim:
                 plt.xlim(xlim)
 
@@ -532,7 +476,7 @@ class VectorSequence(VectorMultiset):
             figure_handles.append(plt.figure(figsize=figsize, facecolor=self.facecolor))
             plt.plot(self.data.loc[:, self.basis_col_name], decomposition.trend, '-', label='Trend', color='green')
             add_labels(xlabel, ylabel)
-            plt.title(f'{title}  // trend', size=self.title_fontsize)
+            plt.title(f'{title}trend', size=self.title_fontsize)
             if xlim:
                 plt.xlim(xlim)
 
@@ -541,7 +485,7 @@ class VectorSequence(VectorMultiset):
             plt.plot(self.data.loc[:, self.basis_col_name], decomposition.seasonal, '-', label='Seasonality',
                      color='darkslateblue')
             add_labels(xlabel, ylabel)
-            plt.title(f"{title} // seasonality with period {period}", size=self.title_fontsize)
+            plt.title(f"{title}seasonality with period {period}", size=self.title_fontsize)
             if xlim:
                 plt.xlim(xlim)
 
@@ -549,7 +493,7 @@ class VectorSequence(VectorMultiset):
             figure_handles.append(plt.figure(figsize=figsize, facecolor=self.facecolor))
             plt.plot(self.data.loc[:, self.basis_col_name], decomposition.resid, '-', label='Residual', color='darkred')
             add_labels(xlabel, ylabel)
-            plt.title(f'{title}  // residual', size=self.title_fontsize)
+            plt.title(f'{title}residual', size=self.title_fontsize)
             if xlim:
                 plt.xlim(xlim)
 
@@ -582,7 +526,7 @@ class VectorSequence(VectorMultiset):
         return InfoSurface(**result.dict())
 
     def plot_info_surface(self, window_widths: List[float] = None, cols: Union[str, List[str]] = None,
-                          singular_values: List[int] = None, *args, **kwargs) -> List[plt.Figure]:
+                          singular_values: List[int] = None, style: Style = None, *args, **kwargs) -> List[plt.Figure]:
         """ Calculates and plots the info surface: value of singular vectors as a function of window start and width
 
         :param window_widths: window widths for the sliding window analysis
@@ -590,10 +534,57 @@ class VectorSequence(VectorMultiset):
         :param singular_values: singular values for which to plot the surface (default: [1, 2, 3])
         :param args: args for sliding window analysis and eval function
         :param kwargs: kwargs for sliding window analysis and eval function
+        :param style: isthmuslib Style object for the colormap
         :return: list of figure handles
         """
-        result: InfoSurface = self.calculate_info_surface(window_widths=window_widths, cols=cols, *args, **kwargs)
-        return result.plot_info_surface(singular_values=singular_values)
+        # Set style. Overrides: kwargs > style input > Style() defaults
+        config: Style = Style(**Style().override(style).dict() | kwargs)
+        svd_kwargs: dict[str, Any] = {k: v for k, v in kwargs.items() if k not in config.dict()}
+        style_kwargs: dict[str, Any] = {k: v for k, v in kwargs.items() if k in config.dict()}
+
+        result: InfoSurface = self.calculate_info_surface(window_widths=window_widths, cols=cols, *args, **svd_kwargs)
+        return result.plot_info_surface(singular_values=singular_values, **style_kwargs)
+
+    def correlation_matrix(self, exclude_basis: bool = True, **kwargs) -> pd.DataFrame:
+        """
+        Very thin wrapper around correlation matrix (using pandas corr() method)
+
+        :param exclude_basis: whether to exclude the basis column from correlation analysis
+        :param kwargs: additional keyword arguments for correlation_matrix (fed through to df.style.background_gradient)
+        :return: styled pandas dataframe
+        """
+        if exclude_basis:
+            if kwargs_in := kwargs.get('exclude_cols'):
+                if self.basis_col_name not in kwargs_in:
+                    kwargs['exclude_cols'] = kwargs_in + [self.basis_col_name]
+            else:
+                kwargs: dict[str, Any] = {'exclude_cols': self.basis_col_name}
+        return correlation_matrix(self.data, **kwargs)
+
+
+def correlation_matrix(dataframe: pd.DataFrame, use_cols: list[str] = None, exclude_cols: list[str] = None,
+                       correlation_method: str = 'pearson', style: Style = None, **kwargs) -> pd.DataFrame:
+    """
+    Returns a styled pandas data frame with correlation coefficients (Pearson by default), wraps pandas.corr()
+
+    :param dataframe: dataframe to analyze
+    :param use_cols: which columns to use
+    :param exclude_cols: which columns to exclude (NB: exclude overrides include)
+    :param correlation_method: Method of correlation {‘pearson’, ‘kendall’, ‘spearman’} or Callable
+    :param style: isthmuslib Style object for the colormap
+    :param kwargs: additional keyword arguments for df.style.background_gradient()
+    :return: styled pandas dataframe
+    """
+    if not use_cols:
+        use_cols: list[str] = dataframe.keys().tolist()
+    if exclude_cols:
+        use_cols: list[str] = [x for x in use_cols if x not in exclude_cols]
+    if style:
+        kwargs.setdefault('cmap', style.cmap)
+    else:
+        kwargs.setdefault('cmap', Style().cmap)
+    corr: pd.DataFrame = dataframe.loc[:, use_cols].corr(method=correlation_method)
+    return corr.style.background_gradient(**kwargs)  # noqa: misinterprets type
 
 
 def info_surface_slider(vs: VectorSequence, *args, **kwargs) -> Dict[str, Any]:
